#  -------------------------------
#  Difficult Rocket
#  Copyright © 2020-2023 by shenjackyuanjie 3695888@qq.com
#  All rights reserved
#  -------------------------------

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from lib_not_dr.nuitka import (
        reader,
        compile
    )

__all__ = [
    "reader",
    "compile"
]
